#!/usr/bin/python
# -*- coding: UTF-8 -*-

import feedparser, sys, webRequests

dynRssProviderAdresses = ("http://www.podnova.com", "http://www.gigadial.com", "http://allpodcasts.com", "http://feeds.feedburner.com/theDailyPodcastFeed-MotherOfAllFeedsOnGigadialPublic");

# checks if url is a valid Podcastfeed
def isFeedValid(url):
  try:
    url = str(url).strip();

    d = feedparser.parse(webRequests.getHttpData(url));
    try:
      title = d.feed.title;
      try:
        entries = d.entries;
        try:
          validEnclosureCount=0;
          for entry in entries:
            try:
              for enclosure in entry.enclosures:
                if enclosure.has_key('url'):
                  validEnclosureCount=validEnclosureCount+1;
            except:
              # this item has no enclosure - checking the rest
              pass
          if validEnclosureCount>0:
            return ("OK", title)
          else:
            return ("ERROR:NOENCLOSURES",title)
        except:
          print "FEEDPARSER-WRAPPER-MOD: isFeedValid: ",sys.exc_info()[0],sys.exc_info()[1]
          return ("ERROR:UNKNOWN:", "")
      except:
        # no items in feed
        return ("ERROR:NOENTRIES",title)
    except:
      # url seems not to be a newsfeed
      return ("ERROR:NOFEED","")
  except:
    # problems on reading url
    return ("ERROR:HTTP","")


# gets PodcastData from Feed
def getPodcastData(url):
  global dynRssProvider;

  podcastData = {};
  itemcount=0;
  try:

    d = feedparser.parse(webRequests.getHttpData(url));
    try:
      feedtitle = d.feed.title;
    except:
      feedtitle = url;
      print "WARNING: Feed has no or unvalid title."

    podcastData['title'] = feedtitle;
    podcastData['url']   = url;
    podcastData['timeorder']   = True;


    podcastitems = [];
    print "feed has the following number of entries:",len(d.entries);
    for entry in d.entries:
      podcastItem = {};

      # podcastitem - title
      podcastItem['feedtitle']=feedtitle;
      try:
        podcastItem['title']=entry.title;
        # print "found title: ",entry.title;
      except:
        podcastItem['title']= "Audioshow no. "+str(itemcount+1)+" in feed";
        print "WARNING: Item has no or unvalid title."

      try:
        if not entry.modified_parsed==None:
          ca_unixtime = ((int(entry.modified_parsed[0])-1970)*60*60*24*365) + (int(entry.modified_parsed[1])*60*60*24*30) + (int(entry.modified_parsed[2])*60*60*24) + (int(entry.modified_parsed[3])*60*60) + (int(entry.modified_parsed[4])*60) + (int(entry.modified_parsed[5]));
        else:
          ca_unixtime=tryToParsePubDate(entry.modified);
        podcastItem['realtimekey']= ca_unixtime;
        podcastItem['timekey']=(ca_unixtime*1000)+(999-itemcount);
      except:
          podcastItem['realtimekey']= 0;
          podcastItem['timekey']=999-itemcount;

          # print pubDate if available
          try:
            print "PROBLEM WITH PubDate:",entry.modified;
          except:
            print "not given";

          # print error message on pubdate
          print "feedparserWrapper:getPodcastData:",sys.exc_info()[0],sys.exc_info()[1]

      # try to get data from enclosure if exists
      try:
        # just take the first enclosure in item
        enclosure_dir=entry.enclosures[0];

        # get enclosure url
        if enclosure_dir.has_key('url'):
          podcastItem['url']    = str(enclosure_dir['url']);
        else:
          podcastItem['url'] = "";

        # get enclosure length
        if enclosure_dir.has_key('length'):
          podcastItem['length'] = str(enclosure_dir['length']);
        else:
          podcastItem['length'] = str(1);

        # make sure length is a integer-value
        try:
          test = int(podcastItem['length']);
        except:
          print "feedWrapper: BAD LENGTH-VALUE - NOT A INTEGER-VAULE:",str(enclosure_dir['length']);
          podcastItem['length'] = "1";

        # get enclosure type
        if enclosure_dir.has_key('type'):
          podcastItem['type']   = str(enclosure_dir['type']);
        else:
          podcastItem['type']   = str("audio");

        # add item to podcastitems
        if not podcastItem['url']=="":
          if podcastItem['realtimekey']<100 :
            print "feedWrapper: Problems on time order.";
            podcastData['timeorder']   = False;
          podcastitems.append(1);
          podcastitems[itemcount]=podcastItem;
          itemcount=itemcount+1;

      except:
        print "> no Enclusures on item :"
        print sys.exc_info()[0], sys.exc_info()[1]

    # podcastitems in podcastData eintragen
    podcastData['items']=podcastitems;
    podcastData['items_count']=itemcount;

    # test if feed is a dynamic rssfeed
    podcastData['dynamicRSS'] = False;
    for dynRssProvider in dynRssProviderAdresses:
      if podcastData['url'].startswith(dynRssProvider):
        podcastData['dynamicRSS'] = True;

    return podcastData
  except:
    print "!! FEEDPROBLEM"
    print "feedparserWrapper:getPodcastdata: General problem with feed:"
    print sys.exc_info()[0], sys.exc_info()[1]
    return False

# sorts all items
# - newest first
# - removes double entries
def sortItems_newestFirst(itemlist):
  newList = [];
  itemlist_backup = itemlist;
  print "Start sort itemlist 'newest files first':";

  try:

    while len(itemlist)>0:
      index=0;
      greatestUnixTime=-1; greatest_index=0;
      while index<len(itemlist):

        try:
          size_at_index = int(itemlist[index]['realtimekey']);
        except:
          print "feedparser:sortItems_newestFirst: Problems with timekeyvaleu on item. Try to manage.";
          size_at_index = 0;
          itemlist[index]['realtimekey']=0;

        if (greatestUnixTime<0) or (size_at_index>greatestUnixTime):
          greatestUnixTime=size_at_index;
          greatest_index=index;
        index=index+1;

      # check if items is already in list // avoid double items
      isAlreadyInList = 0;
      for itemInList in newList:
        if itemInList['url']==itemlist[greatest_index]['url']:
          isAlreadyInList = 1;
          try:
            title = itemlist[greatest_index]['title'].encode('MBCS');
            print "found double entry for item:",str(title);
          except:
            print "found double entry for item: (title not displayable)";              

      if isAlreadyInList==0:
        # add item to new list
        newList=newList+[itemlist[greatest_index],];
        try:
          title = itemlist[greatest_index]['title'].encode('MBCS');
          print str(itemlist[greatest_index]['realtimekey'])+" / "+str(title);
        except:
          print str(itemlist[greatest_index]['realtimekey'])+" / (title not displayable)";            

      # delete item in old list
      del itemlist[greatest_index];

  except:
    print "feedparserWrapper: Problems on sorting newest items:",sys.exc_info()[0],sys.exc_info()[1]
    newList = itemlist_backup;

  return newList;

# try to parse a unixtime from string
def tryToParsePubDate(pubstr):
  try:
    year   = 0;
    day    = 0;
    month  = 0;
    hour   = 0;
    min    = 0;
    sec    = 0;

    pubstr = str(pubstr);

    if len(pubstr)==0:
      return 0;

    if pubstr.find("/",0,len(pubstr)):
      parts = pubstr.split(" ",3);

      dateStr = parts[0];
      dparts = dateStr.split("/",3);
      month = int(dparts[0]);
      day   = int(dparts[1]);
      year  = int(dparts[2]);

      if len(parts)>1:
        timeStr = parts[1];
        tparts = timeStr.split(":",3);
        hour = int(tparts[0]);
        min  = int(tparts[1]);
        sec  = int(tparts[2]);

      if len(parts)>2:
        if parts[2]=="PM":
          hour=hour+12;

    ca_unixtime =              ((int(year)-1970)*60*60*24*365);
    ca_unixtime = ca_unixtime + (int(month)*60*60*24*30);
    ca_unixtime = ca_unixtime + (int(day)*60*60*24);
    ca_unixtime = ca_unixtime + (int(hour)*60*60);
    ca_unixtime = ca_unixtime + (int(min)*60);
    ca_unixtime = ca_unixtime + (int(sec));

    return ca_unixtime;

  except:
    print "feedWrapper:tryToParsePubDate: Error on parsing pubDate manually:",sys.exc_info()[0], sys.exc_info()[1];
    return 0;